<?php

declare(strict_types=1);

namespace Dedi\SyliusSettingsPlugin\Form\Type;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\Contracts\SyliusPluginSettings\SettingsProviderRegistryInterface;
use MonsieurBiz\SyliusSettingsPlugin\Form\AbstractSettingsType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\FormBuilderInterface;

class SyliusPluginsSettingsType extends AbstractSettingsType
{
    public function __construct(private readonly SettingsProviderRegistryInterface $settingsProviderRegistry)
    {
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        /** @var SettingsProviderInterface $settingsProvider */
        foreach ($this->settingsProviderRegistry->getAll() as $settingsProvider) {
            $isPluginEnabledByDefaultConfig = $settingsProvider->provide(true)->isEnabled();
            $isPluginEnabledByUserConfig = $settingsProvider->provide()->isEnabled();

            if (true === $isPluginEnabledByDefaultConfig) {
                $pluginName = $settingsProvider->getName();
                $this->addWithDefaultCheckbox(
                    $builder,
                    $pluginName,
                    CheckboxType::class,
                    [
                        'label' => 'dedi.settings.' . $pluginName . '.form_label',
                        'required' => false,
                        'data' => $isPluginEnabledByUserConfig,
                    ],
                );
            }
        }
    }
}
