<?php

declare(strict_types=1);

namespace Dedi\SyliusSettingsPlugin\Provider;

use Dedi\Contracts\SyliusPluginSettings\SettingsFactoryInterface;
use Dedi\Contracts\SyliusPluginSettings\SettingsInterface;
use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\SyliusSettingsPlugin\Model\Config;
use MonsieurBiz\SyliusSettingsPlugin\Provider\SettingsProviderInterface as MonsieurBizSettingsProviderInterface;
use Symfony\Component\PropertyAccess\PropertyAccessorInterface;
use Symfony\Component\Serializer\NameConverter\CamelCaseToSnakeCaseNameConverter;

class Decorator implements SettingsProviderInterface
{
    public function __construct(
        private readonly SettingsProviderInterface $decoratedSettingsProvider,
        private readonly SettingsFactoryInterface $settingsFactory,
        private readonly MonsieurBizSettingsProviderInterface $monsieurBizSettingsProvider,
        private readonly PropertyAccessorInterface $propertyAccessor,
        private readonly Config $config,
    ) {
    }

    public function getName(): string
    {
        return $this->decoratedSettingsProvider->getName();
    }

    public function provide(bool $default = false): SettingsInterface
    {
        $defaultSettings = $this->decoratedSettingsProvider->provide($default);

        if ($default) {
            return $defaultSettings;
        }

        $isPluginEnabledByUser = $this->monsieurBizSettingsProvider->getSettingValue('dedi.sylius_plugins_settings', $this->decoratedSettingsProvider->getName());
        $isPluginEnabledByDefaultConfig = $defaultSettings->isEnabled();

        if (null === $isPluginEnabledByUser) {
            $isPluginEnabledByUser = $isPluginEnabledByDefaultConfig;
        }

        if (false !== $isPluginEnabledByDefaultConfig) {
            $keys = $this->config->getKeys();
            $options = [];

            foreach ($keys as $key) {
                $camelCasedKey = (new CamelCaseToSnakeCaseNameConverter())->denormalize($key);
                $options[$camelCasedKey] = $this->monsieurBizSettingsProvider->getSettingValue('app.' . $this->decoratedSettingsProvider->getName(), $key) ?? $this->propertyAccessor->getValue($defaultSettings, $key);
            }

            $options['enabled'] = $isPluginEnabledByUser;

            return $this
                ->settingsFactory
                ->create($this->config->getModel(), $options);
        }

        return $defaultSettings;
    }
}
